@@ -55,16 +55,36 @@ class NoConfigFileReadError(Error):
 _UNSET = object()
 
 
+class Container(ABC):
+    """Abstract Mixin Class
+    """
+    def __init__(self, **kwargs):
+        self._structure = list()
+        super().__init__(**kwargs)
+
+    @property
+    def structure(self):
+        return self._structure
+
+    @property
+    def last_item(self):
+        if self._structure:
+            return self._structure[-1]
+        else:
+            return None
+
+
 class Block(ABC):
     """Abstract Block type holding lines
 
     Block objects hold original lines from the configuration file and hold
     a reference to a container wherein the object resides.
     """
-    def __init__(self, container=None):
+    def __init__(self, container=None, **kwargs):
         self._container = container
         self.lines = []
         self._updated = False
+        super().__init__(**kwargs)
 
     def __str__(self):
         return ''.join(self.lines)
@@ -74,38 +94,47 @@ class Block(ABC):
 
     def __eq__(self, other):
         if isinstance(other, self.__class__):
-            return self.lines == other.lines
+            return str(self) == str(other)
         else:
             return False
 
-    def _add_line(self, line):
+    def add_line(self, line):
+        """Add a line to the current block
+
+        Args:
+            line (str): one line to add
+        """
         self.lines.append(line)
+        return self
+
+    @property
+    def container(self):
+        return self._container
 
     @property
     def add_before(self):
         """Returns a builder inserting a new block before the current block"""
-        idx = self._container.index(self)
-        return BlockBuilder(self._container, type(self), idx)
+        idx = self._container.structure.index(self)
+        return BlockBuilder(self._container, idx)
 
     @property
     def add_after(self):
         """Returns a builder inserting a new block after the current block"""
-        idx = self._container.index(self)
-        return BlockBuilder(self._container, type(self), idx+1)
+        idx = self._container.structure.index(self)
+        return BlockBuilder(self._container, idx+1)
 
 
 class BlockBuilder(object):
     """Builder that injects blocks at a given index position."""
-    def __init__(self, container, block_type, idx):
+    def __init__(self, container, idx):
         self._container = container
-        self._block_type = block_type
         self._idx = idx
 
     def comment(self, text, comment_prefix='#'):
         """Creates a comment block
 
         Args:
-            text (str): content of comment without \#
+            text (str): content of comment without #
             comment_prefix (str): character indicating start of comment
 
         Returns:
@@ -114,10 +143,10 @@ class BlockBuilder(object):
         comment = Comment(self._container)
         if not text.startswith(comment_prefix):
             text = "{} {}".format(comment_prefix, text)
-        if not text.endswith(os.linesep):
-            text = "{}{}".format(text, os.linesep)
-        comment._add_line(text)
-        self._container.insert(self._idx, comment)
+        if not text.endswith('\n'):
+            text = "{}{}".format(text, '\n')
+        comment.add_line(text)
+        self._container.structure.insert(self._idx, comment)
         self._idx += 1
         return self
 
@@ -130,7 +159,7 @@ class BlockBuilder(object):
         Returns:
             self for chaining
         """
-        if self._block_type is not Section:
+        if not isinstance(self._container, ConfigUpdater):
             raise ValueError("Sections can only be added at section level!")
         if isinstance(section, str):
             # create a new section
@@ -140,7 +169,7 @@ class BlockBuilder(object):
         if section.name in [block.name for block in self._container
                             if isinstance(block, Section)]:
             raise DuplicateSectionError(section.name)
-        self._container.insert(self._idx, section)
+        self._container.structure.insert(self._idx, section)
         self._idx += 1
         return self
 
@@ -155,8 +184,8 @@ class BlockBuilder(object):
         """
         space = Space()
         for line in range(newlines):
-            space._add_line(os.linesep)
-        self._container.insert(self._idx, space)
+            space.add_line('\n')
+        self._container.structure.insert(self._idx, space)
         self._idx += 1
         return self
 
@@ -171,11 +200,13 @@ class BlockBuilder(object):
         Returns:
             self for chaining
         """
-        if self._block_type is not Option:
+        if not isinstance(self._container, Section):
             raise ValueError("Options can only be added inside a section!")
         option = Option(key, value, container=self._container, **kwargs)
+        if option.key in self._container.options():
+            raise DuplicateOptionError(self._container.name, option.key)
         option.value = value
-        self._container.insert(self._idx, option)
+        self._container.structure.insert(self._idx, option)
         self._idx += 1
         return self
 
@@ -183,7 +214,7 @@ class BlockBuilder(object):
 class Comment(Block):
     """Comment block"""
     def __init__(self, container=None):
-        super().__init__(container)
+        super().__init__(container=container)
 
     def __repr__(self):
         return '<Comment>'
@@ -192,45 +223,63 @@ class Comment(Block):
 class Space(Block):
     """Vertical space block of new lines"""
     def __init__(self, container=None):
-        super().__init__(container)
+        super().__init__(container=container)
 
     def __repr__(self):
         return '<Space>'
 
 
-class Section(Block, MutableMapping):
+class Section(Block, Container, MutableMapping):
     """Section block holding options
 
     Attributes:
         name (str): name of the section
         updated (bool): indicates name change or a new section
     """
-    def __init__(self, name, container):
+    def __init__(self, name, container, **kwargs):
         self._name = name
         self._structure = list()
         self._updated = False
-        super().__init__(container)
+        super().__init__(container=container, **kwargs)
 
-    # used when constructing the section
-    def _add_option(self, entry):
+    def add_option(self, entry):
+        """Add an Option object to the section
+
+        Used during initial parsing mainly
+
+        Args:
+            entry (Option): key value pair as Option object
+        """
         self._structure.append(entry)
+        return self
 
-    def _add_comment(self):
-        if not isinstance(self._curr_entry, Comment):
+    def add_comment(self, line):
+        """Add a Comment object to the section
+
+        Used during initial parsing mainly
+
+        Args:
+            line (str): one line in the comment
+        """
+        if not isinstance(self.last_item, Comment):
             comment = Comment(self._structure)
             self._structure.append(comment)
+        self.last_item.add_line(line)
+        return self
 
-    def _add_space(self):
-        if not isinstance(self._curr_entry, Space):
+    def add_space(self, line):
+        """Add a Space object to the section
+
+        Used during initial parsing mainly
+
+        Args:
+            line (str): one line that defines the space, maybe whitespaces
+        """
+        if not isinstance(self.last_item, Space):
             space = Space(self._structure)
             self._structure.append(space)
-
-    @property
-    def _curr_entry(self):
-        if self._structure:
-            return self._structure[-1]
-        else:
-            return None
+        self.last_item.add_line(line)
+        return self
 
     def _get_option_idx(self, key):
         idx = [i for i, entry in enumerate(self._structure)
@@ -253,16 +302,17 @@ class Section(Block, MutableMapping):
         return '<Section: {}>'.format(self.name)
 
     def __getitem__(self, key):
+        key = self._container.optionxform(key)
         if key not in self.options():
             raise KeyError(key)
         return self._structure[self._get_option_idx(key=key)]
 
     def __setitem__(self, key, value):
-        if key in self:
+        if self._container.optionxform(key) in self:
             option = self.__getitem__(key)
             option.value = value
         else:
-            option = Option(key, value, container=self._structure)
+            option = Option(key, value, container=self)
             option.value = value
             self._structure.append(option)
 
@@ -329,6 +379,28 @@ class Section(Block, MutableMapping):
         self._name = str(value)
         self._updated = True
 
+    def set(self, option, value=None):
+        """Set an option for chaining.
+
+        Args:
+            option (str): option name
+            value (str): value, default None
+        """
+        option = self._container.optionxform(option)
+        if option in self.options():
+            self.__getitem__(option).value = value
+        else:
+            self.__setitem__(option, value)
+        return self
+
+    def insert_at(self, idx):
+        """Returns a builder inserting a new block at the given index
+
+        Args:
+            idx (int): index where to insert
+        """
+        return BlockBuilder(self, idx)
+
 
 class Option(Block):
     """Option block holding a key/value pair.
@@ -340,7 +412,7 @@ class Option(Block):
     """
     def __init__(self, key, value, container, delimiter='=',
                  space_around_delimiters=True, line=None):
-        super().__init__(container)
+        super().__init__(container=container)
         self._key = key
         self._values = [value]
         self._value_is_none = value is None
@@ -352,28 +424,28 @@ class Option(Block):
         if line:
             self.lines.append(line)
 
-    def _add_line(self, line):
-        super()._add_line(line)
+    def add_line(self, line):
+        super().add_line(line)
         self._values.append(line.strip())
 
     def _join_multiline_value(self):
         if not self._multiline_value_joined and not self._value_is_none:
             # do what `_join_multiline_value` in ConfigParser would do
-            self._value = os.linesep.join(self._values).rstrip()
+            self._value = '\n'.join(self._values).rstrip()
             self._multiline_value_joined = True
 
     def __str__(self):
         if not self.updated:
             return super().__str__()
         if self._value is None:
-            return "{}{}".format(self._key, os.linesep)
+            return "{}{}".format(self._key, '\n')
         if self._space_around_delimiters:
             # no space is needed if we use multi-line arguments
-            suffix = '' if str(self._value).startswith(os.linesep) else ' '
+            suffix = '' if str(self._value).startswith('\n') else ' '
             delim = " {}{}".format(self._delimiter, suffix)
         else:
             delim = self._delimiter
-        return "{}{}{}{}".format(self._key, delim, self._value, os.linesep)
+        return "{}{}{}{}".format(self._key, delim, self._value, '\n')
 
     def __repr__(self):
         return '<Option: {} = {}>'.format(self.key, self.value)
@@ -386,7 +458,7 @@ class Option(Block):
 
     @property
     def key(self):
-        return self._key
+        return self._container._container.optionxform(self._key)
 
     @key.setter
     def key(self, value):
@@ -406,7 +478,7 @@ class Option(Block):
         self._value = value
         self._values = [value]
 
-    def set_values(self, values, separator=os.linesep, indent=4*' '):
+    def set_values(self, values, separator='\n', indent=4*' '):
         """Sets the value to a given list of options, e.g. multi-line values
 
         Args:
@@ -417,13 +489,13 @@ class Option(Block):
         self._updated = True
         self._multiline_value_joined = True
         self._values = values
-        if separator == os.linesep:
+        if separator == '\n':
             values.insert(0, '')
             separator = separator + indent
         self._value = separator.join(values)
 
 
-class ConfigUpdater(MutableMapping):
+class ConfigUpdater(Container, MutableMapping):
     """Parser for updating configuration files.
 
     ConfigUpdater follows the API of ConfigParser with some differences:
@@ -515,13 +587,7 @@ class ConfigUpdater(MutableMapping):
         self._allow_no_value = allow_no_value
         # Options from ConfigParser that we need to set constantly
         self._empty_lines_in_values = False
-
-    @property
-    def _curr_block(self):
-        if self._structure:
-            return self._structure[-1]
-        else:
-            return None
+        super().__init__()
 
     def _get_section_idx(self, name):
         idx = [i for i, entry in enumerate(self._structure)
@@ -545,15 +611,17 @@ class ConfigUpdater(MutableMapping):
     def read_file(self, f, source=None):
         """Like read() but the argument must be a file-like object.
 
-        The `f` argument must be iterable, returning one line at a time.
-        Optional second argument is the `source` specifying the name of the
-        file being read. If not given, it is taken from f.name. If `f` has no
-        `name` attribute, `<???>` is used.
+        The ``f`` argument must be iterable, returning one line at a time.
+        Optional second argument is the ``source`` specifying the name of the
+        file being read. If not given, it is taken from f.name. If ``f`` has no
+        ``name`` attribute, ``<???>`` is used.
 
         Args:
             f: file like object
             source (str): reference name for file object, default None
         """
+        if isinstance(f, str):
+            raise RuntimeError("f must be a file-like object, not string!")
         if source is None:
             try:
                 source = f.name
@@ -575,47 +643,45 @@ class ConfigUpdater(MutableMapping):
         """Converts an option key to lower case for unification
 
         Args:
-            optionstr (str): key name
+             optionstr (str): key name
 
         Returns:
-            str: key name
+            str: unified option name
         """
         return optionstr.lower()
 
     def _update_curr_block(self, block_type):
-        if not isinstance(self._curr_block, block_type):
-            new_block = block_type(container=self._structure)
+        if not isinstance(self.last_item, block_type):
+            new_block = block_type(container=self)
             self._structure.append(new_block)
 
     def _add_comment(self, line):
-        if isinstance(self._curr_block, Section):
-            self._curr_block._add_comment()
-            self._curr_block._curr_entry._add_line(line)
+        if isinstance(self.last_item, Section):
+            self.last_item.add_comment(line)
         else:
             self._update_curr_block(Comment)
-            self._curr_block._add_line(line)
+            self.last_item.add_line(line)
 
     def _add_section(self, sectname, line):
-        new_section = Section(sectname, container=self._structure)
-        new_section._add_line(line)
+        new_section = Section(sectname, container=self)
+        new_section.add_line(line)
         self._structure.append(new_section)
 
     def _add_option(self, key, vi, value, line):
         entry = Option(
             key, value,
             delimiter=vi,
-            container=self._curr_block._structure,
+            container=self.last_item,
             space_around_delimiters=self._space_around_delimiters,
             line=line)
-        self._curr_block._add_option(entry)
+        self.last_item.add_option(entry)
 
     def _add_space(self, line):
-        if isinstance(self._curr_block, Section):
-            self._curr_block._add_space()
-            self._curr_block._curr_entry._add_line(line)
+        if isinstance(self.last_item, Section):
+            self.last_item.add_space(line)
         else:
             self._update_curr_block(Space)
-            self._curr_block._add_line(line)
+            self.last_item.add_line(line)
 
     def _read(self, fp, fpname):
         """Parse a sectioned configuration file.
@@ -678,7 +744,7 @@ class ConfigUpdater(MutableMapping):
                             optname and
                             cursect[optname] is not None):
                         cursect[optname].append('')  # newlines added at join
-                        self._curr_block._curr_entry._add_line(line)  # HOOK
+                        self.last_item.last_item.add_line(line)  # HOOK
                 else:
                     # empty line marks end of value
                     indent_level = sys.maxsize
@@ -691,7 +757,7 @@ class ConfigUpdater(MutableMapping):
             if (cursect is not None and optname and
                     cur_indent_level > indent_level):
                 cursect[optname].append(value)
-                self._curr_block._curr_entry._add_line(line)  # HOOK
+                self.last_item.last_item.add_line(line)  # HOOK
             # a section header or option header?
             else:
                 indent_level = cur_indent_level
@@ -722,7 +788,9 @@ class ConfigUpdater(MutableMapping):
                         optname, vi, optval = mo.group('option', 'vi', 'value')
                         if not optname:
                             e = self._handle_error(e, fpname, lineno, line)
-                        optname = self.optionxform(optname.rstrip())
+                        # optname = self.optionxform(optname.rstrip())
+                        # keep original case of key
+                        optname = optname.rstrip()
                         if (self._strict and
                                 (sectname, optname) in elements_added):
                             raise DuplicateOptionError(sectname, optname,
@@ -753,21 +821,29 @@ class ConfigUpdater(MutableMapping):
         exc.append(lineno, repr(line))
         return exc
 
-    def write(self, fp):
+    def write(self, fp, validate=True):
         """Write an .ini-format representation of the configuration state.
 
         Args:
             fp (file-like object): open file handle
+            validate (Boolean): validate format before writing
         """
+        if validate:
+            self.validate_format()
         fp.write(str(self))
 
-    def update_file(self):
+    def update_file(self, validate=True):
         """Update the read-in configuration file.
+
+        Args:
+            validate (Boolean): validate format before writing
         """
         if self._filename is None:
             raise NoConfigFileReadError()
+        if validate:  # validate BEFORE opening the file!
+            self.validate_format()
         with open(self._filename, 'w') as fb:
-            self.write(fb)
+            self.write(fb, validate=False)
 
     def validate_format(self, **kwargs):
         """Call ConfigParser to validate config
@@ -861,7 +937,7 @@ class ConfigUpdater(MutableMapping):
             raise DuplicateSectionError(section)
         if isinstance(section, str):
             # create a new section
-            section = Section(section, container=self._structure)
+            section = Section(section, container=self)
         elif not isinstance(section, Section):
             raise ValueError("Parameter must be a string or Section type!")
         self._structure.append(section)
@@ -964,6 +1040,7 @@ class ConfigUpdater(MutableMapping):
             section[option].value = value
         else:
             section[option] = value
+        return self
 
     def remove_option(self, section, option):
         """Remove an option.
