@@ -279,6 +279,28 @@ def test_set_option():
         updater.set('wrong_section', 'key', '1')
 
 
+def test_section_set_option():
+    updater = ConfigUpdater()
+    updater.read_string(test1_cfg_in)
+    default_sec = updater['default']
+    default_sec.set('key', '1')
+    assert default_sec['key'].value == '1'
+    default_sec.set('key', 2)
+    assert default_sec['key'].value == 2
+    assert str(default_sec) == test1_cfg_out[1:]
+    default_sec.set('key')
+    assert default_sec['key'].value is None
+    assert str(default_sec) == test1_cfg_out_none[1:]
+    updater.read_string(test1_cfg_in)
+    default_sec = updater['default']
+    default_sec.set('other_key', 3)
+    assert str(default_sec) == test1_cfg_out_added[1:]
+    updater.read_string(test1_cfg_in)
+    default_sec = updater['default']
+    default_sec['key'].set_values(['param1', 'param2'])
+    assert str(default_sec) == test1_cfg_out_values[1:]
+
+
 def test_del_option():
     updater = ConfigUpdater()
     updater.read_string(test1_cfg_in)
@@ -607,6 +629,7 @@ def test_eq(setup_cfg_path):
     assert updater1 != updater2
     assert updater1 != updater2['metadata']
     assert updater2['metadata'] != updater2['metadata']['author']
+    assert not updater1.remove_section('metadata')
 
 
 test12_cfg_in = """
@@ -625,3 +648,144 @@ def test_rename_option_key():
     updater = ConfigUpdater()
     updater.read_string(test12_cfg_in)
     updater['section']['opiton'].key = 'option'
+
+
+test13_cfg_in = """
+[section]
+CAPITAL = 1
+"""
+
+
+def test_set_optionxform():
+    updater = ConfigUpdater()
+    updater.read_string(test13_cfg_in)
+    assert updater['section']['capital'].value == '1'
+    assert callable(updater.optionxform)
+    updater.optionxform = lambda x: x
+    updater.read_string(test13_cfg_in)
+    assert updater['section']['CAPITAL'].value == '1'
+
+
+test14_cfg_in = """
+[section]
+option2 = 2
+option4 = 4
+"""
+
+test14_cfg_out = """
+[section]
+option0 = 0
+option1 = 1
+option2 = 2
+option3 = 3
+option4 = 4
+"""
+
+
+def test_insert_at():
+    updater = ConfigUpdater()
+    updater.read_string(test14_cfg_in)
+    updater['section'].insert_at(0).option("option0", 0).option("option1", 1)
+    updater['section'].insert_at(3).option("option3", 3)
+    assert str(updater) == test14_cfg_out
+
+
+def test_read_file_with_string():
+    updater = ConfigUpdater()
+    with pytest.raises(RuntimeError):
+        updater.read_file('path/to/file.cfg')
+
+
+test15_cfg_in = """
+[section]
+OptionA = 2
+"""
+
+
+def test_read_mixed_case_options():
+    updater = ConfigUpdater()
+    updater.read_string(test15_cfg_in)
+    assert updater.has_option('section', 'OptionA')
+    assert updater.has_option('section', 'optiona')
+    assert updater['section']['OptionA'].value == '2'
+    assert updater['section']['optiona'].value == '2'
+
+
+test16_cfg_in = """
+[section]
+OptionA = 2
+"""
+
+
+test16_cfg_out = """
+[section]
+OptionA = 4
+OptionB = 6
+"""
+
+
+def test_update_mixed_case_options():
+    updater = ConfigUpdater()
+    updater.read_string(test16_cfg_in)
+    updater['section']['optiona'].value = '4'
+    updater['section']['OptionB'] = '6'
+    assert str(updater) == test16_cfg_out
+
+
+test17_cfg_in = """
+[section]
+key1 = 1
+"""
+
+
+test17_cfg_out = """
+[section]
+key0 = 0
+key1 = 1
+key2 = 2
+key3 = 3
+"""
+
+
+def test_add_before_then_add_after_option():
+    updater = ConfigUpdater()
+    updater.read_string(test17_cfg_in)
+    updater['section']['key1'].add_before.option('key0', '0')
+    updater['section']['key1'].add_after.option('key2', '2')
+    updater['section']['key2'].add_after.option('key3', '3')
+    assert str(updater) == test17_cfg_out
+
+
+test18_cfg_in = """
+[section]
+Key0 = 0
+"""
+
+test18_cfg_out = """
+[section]
+Key0 = 2
+"""
+
+
+def test_assure_no_duplicate_options():
+    updater = ConfigUpdater()
+    updater.read_string(test18_cfg_in)
+    updater['section']['KEY0'].value = '1'
+    updater['section']['keY0'] = '2'
+    assert str(updater) == test18_cfg_out
+
+
+test19_cfg_in = """
+[section]
+Key0 = 0
+"""
+
+
+def test_no_duplicate_blocks_with_blockbuilder():
+    updater = ConfigUpdater()
+    updater.read_string(test19_cfg_in)
+    with pytest.raises(DuplicateOptionError):
+        updater['section']['Key0'].add_after.option('key0', '1')
+    with pytest.raises(DuplicateSectionError):
+        updater['section'].add_after.section('section')
+    assert str(updater) == test19_cfg_in
